from a7585d.a7585d import A7585D 
from a7585d.a7585d import A7585D_REG
import time

#recognize module by serial number so windows assigned port is not relevant

#Serial Number 1 = 28885
#Serial Number 2 = 12297
SNS = [ 28885, 12297 ]

MAP = [0 for i in range(len(SNS))]

hvs = [A7585D() for i in range(len(SNS))]

for i in range(len(SNS)):
    hvs[i].open("COM" + str(i+3))

for i in range(len(SNS)):
    current_sn = hvs[i].get_parameter(A7585D_REG.SERIAL_NUMBER)

    # If the read serial number is in the list, map it
    if current_sn in SNS:
        MAP[SNS.index(current_sn)] = i


print(MAP)


print(hvs[MAP[0]].get_parameter(A7585D_REG.SERIAL_NUMBER))
print(hvs[MAP[1]].get_parameter(A7585D_REG.SERIAL_NUMBER))


import serial
import time
class A7585D_REG:
    HV_ENABLE = 0
    CNTRL_MODE = 1
    V_TARGET = 2
    RAMP = 3
    MAX_V = 4
    MAX_I = 5
    MAX_T = 6
    CAL_TEMP_M2 = 7
    CAL_TEMP_M = 8
    CAL_TEMP_Q = 9
    ALFA_VOUT = 10
    ALFA_IOUT = 11
    ALFA_VREF = 12
    ALFA_TREF = 13
    T_COEF_SIPM = 28
    LUT_ENABLE = 29
    ENABLE_PI = 30
    EMERGENCY_STOP = 31
    IZERO = 32
    LUT_ADDRESS = 36
    LUT_PROGRAM_V = 37
    LUT_PROGRAM_T = 38
    LUT_LENGTH = 39
    I2C_BASE_ADDRESS = 40
    CURRENT_RANGE = 81

    MON_VIN=230
    MON_VOUT=231
    MON_IOUT=232
    MON_VREF=233
    MON_TREF=234
    MON_VTARGET=235
    MON_RTARGET=236
    MON_CVT=237
    MON_COMPLIANCE_V = 249
    MON_COMPLIANCE_I = 250

    PRODUCT_CODE=251
    FW_VERSION=252
    HW_VERSION=253
    SERIAL_NUMBER=254

class A7585D:
    def __init__(self):
        self.ser=None

    def open(self, serial_port):
        self.ser = serial.Serial(serial_port, 115200, timeout=1)
        self.ser.write(b"\r\n")
        self.ser.write(b"\r\n")
        time.sleep(0.1)
        send_string = "AT+MACHINE\r\n"
        self.ser.write(bytes(send_string.encode("utf-8")))
        time.sleep(0.1)
        self.ser.flushInput()
        self.ser.flushOutput()
        time.sleep(1)
    
    def set_parameter(self, param, value):
        send_string = "AT+SET," + str(param) + "," + str("{:.9f}".format(value)) +"\r\n"
        self.ser.write(bytes(send_string.encode("utf-8")))
        res = str(self.ser.readline())
        time.sleep(0.01)
        if "OK" in res:
            return True
        else:
            raise Exception('set', 'error')        


    def get_parameter(self, param):
        send_string = "AT+GET," + str(int(param)) + "\r\n"
        self.ser.write(bytes(send_string.encode("utf-8")))
        res = str(self.ser.readline(),'ascii')
        if "OK" in res:
            a=res.split("=")
            ab=a[1].replace("\\r","").replace("\\n","")
            return float(ab)
        else:
            raise Exception('get', 'error')       

    def save_parameter(self):
        send_string = "AT+SET,255,1\r\n"
        self.ser.write(bytes(send_string.encode("utf-8")))
        time.sleep(2)
        res = str(self.ser.readline())
        if "OK" in res:
            return True
        else:
            raise Exception('set', 'error') 

    def close(self):
        self.ser.close()


from a7585d.a7585d import A7585D 
from a7585d.a7585d import A7585D_REG
import time


hv = A7585D()


# open serial port
hv.open("COM4") # Windows
# hv.open("/dev/ttyUSB0") # Linux USB
# hv.open("/dev/ttyS0") # Linux UART

# configure parameters HV

# set control output control loop mode
# 0 Digital mode (output voltage is vtarget)
# 1 Analog mode (output voltage is proportional to vref)
# 2 Thermal compensation (output voltage is vtarget - Tcoef * (T - 25)
hv.set_parameter(A7585D_REG.CNTRL_MODE, 0)  

# set voltage target to 40V
hv.set_parameter(A7585D_REG.V_TARGET, 42.5)

# set max voltage 1mA
hv.set_parameter(A7585D_REG.MAX_I, 4)

# set max voltage (compliance) to 50V
hv.set_parameter(A7585D_REG.MAX_V, 50)

# configure SiPM temperature compensation coefficient
hv.set_parameter(A7585D_REG.T_COEF_SIPM, -35)

# configure ramp speed to 5V/s
hv.set_parameter(A7585D_REG.RAMP, 5)

# set current monitor range
# 0 low range
# 1 high range
# 2 auto select
hv.set_parameter(A7585D_REG.CURRENT_RANGE, 2)


# control pi controller (1 enable)
hv.set_parameter(A7585D_REG.ENABLE_PI, 0)

# enable hv
hv.set_parameter(A7585D_REG.HV_ENABLE, 0)

while True:
    print("HV V_OUT: " + str(hv.get_parameter(A7585D_REG.MON_VOUT)))
    print("HV I_OUT: " + str(hv.get_parameter(A7585D_REG.MON_IOUT)))
    time.sleep(0.1)



import argparse
from a7585d.a7585d import A7585D
from a7585d.a7585d import A7585D_REG
import time

def main(args):
    hv = A7585D()
    hv.open(args.port)

    print("Module serial number: " + str(hv.get_parameter(A7585D_REG.SERIAL_NUMBER)))
    print("firmware version: " + str(hv.get_parameter(A7585D_REG.FW_VERSION)))
    print("hardware version: " + str(hv.get_parameter(A7585D_REG.HW_VERSION)))

    if args.power_on is not None:
        hv.set_parameter(A7585D_REG.HV_ENABLE, args.power_on)

    if args.control_mode is not None:
        hv.set_parameter(A7585D_REG.CNTRL_MODE, args.control_mode)

    if args.voltage_target is not None:
        hv.set_parameter(A7585D_REG.V_TARGET, args.voltage_target)

    if args.max_current is not None:
        hv.set_parameter(A7585D_REG.MAX_I, args.max_current)

    if args.max_voltage is not None:
        hv.set_parameter(A7585D_REG.MAX_V, args.max_voltage)

    if args.temp_coef is not None:
        hv.set_parameter(A7585D_REG.T_COEF_SIPM, args.temp_coef)

    if args.ramp_speed is not None:
        hv.set_parameter(A7585D_REG.RAMP, args.ramp_speed)

    if args.current_range is not None:
        hv.set_parameter(A7585D_REG.CURRENT_RANGE, args.current_range)

    if args.enable_pi is not None:
        hv.set_parameter(A7585D_REG.ENABLE_PI, args.enable_pi)

    if args.monitor:
        if args.poll_mode == 'continuous':
            while True:
                print("V: " + str(hv.get_parameter(A7585D_REG.MON_VOUT)) + " I: " + str(hv.get_parameter(A7585D_REG.MON_IOUT)))
                time.sleep(0.1)
        else:
            print("V: " + str(hv.get_parameter(A7585D_REG.MON_VOUT)) + " I: " + str(hv.get_parameter(A7585D_REG.MON_IOUT)))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='A7585D HV Control Utility')
    parser.add_argument('--port', type=str, required=True, help='Serial port')
    parser.add_argument('--power-on', type=int, choices=[0, 1], help='Power on HV')
    parser.add_argument('--control-mode', type=int, choices=[0, 1, 2], help='Control mode')
    parser.add_argument('--voltage-target', type=float, help='Voltage target')
    parser.add_argument('--max-current', type=int, help='Max current')
    parser.add_argument('--max-voltage', type=int, help='Max voltage')
    parser.add_argument('--temp-coef', type=int, help='SiPM temperature coefficient')
    parser.add_argument('--ramp-speed', type=int, help='Ramp speed')
    parser.add_argument('--current-range', type=int, choices=[0, 1, 2], help='Current range')
    parser.add_argument('--enable-pi', type=int, choices=[0, 1], help='Enable PI controller')
    parser.add_argument('--poll-mode', type=str, default='one-shot', choices=['one-shot', 'continuous'], help='Reading mode: one-shot or continuous (default: one-shot)')
    parser.add_argument('--monitor', action='store_true', help='Enable monitoring of HV output')
    
    args = parser.parse_args()
    main(args)
